/**********************************************************
* CETPEXT v1.0 (March 19th 2003)                          *
* CETus Position EXTractor                                *
* --------------------------------------------------------*
* Convert Cetus Position (http://www.cetus.dk) PDB file   *
* into an easy readable data file                         *
* --------------------------------------------------------*
* Author:  Matteo Lenardon (e-mail: hangar18@virgilio.it) *
* License: Freeware                                       *
* --------------------------------------------------------*
* v1.0 03/11/2003                                         *
* -1st release                                            *
* v1.0 03/19/2003                                         *
* - csv, cvs correction                                   *
**********************************************************/

/********************** <<INCLUDE>> **********************/
#include <stdio.h>
#include <string.h>

/********************** <<DEFINE>> ***********************/
#define NO_PAR   1
#define SRC_NAME 0
#define DST_NAME 1

/********************* <<TYPEDEF>> ***********************/

/*** 4Byte: long ***/
typedef union
{
  unsigned char Byte[4];
  long All;
} LongByte;

/*** 4Byte: unsigned long ***/
typedef union
{
  unsigned char Byte[4];
  unsigned long	All;
} ULongByte;

/*** 2Byte: int ***/
typedef union
{
  unsigned char Byte[2];
  int All;
} IntByte;

/*** 2Byte: unsigned int ***/
typedef union
{
  unsigned char Byte[2];
  unsigned int All;
} UIntByte;

/*** PDB Header ***/
typedef struct
{
    char strName[33];
    int	 intAttr;
    int	 intVers;
    long lngCreate;
    long lngMod;
    long lngBackUp;
    long lngModNum;
    long lngAppInfoArea;
    long lngSortArea;
    char strType[5];
    char strCreator[5];
    long lngSeed;
    long lngNextRec;
    int	 intNumRecs;
} pdbHeader;


/*** PDB Record Info ***/
typedef struct
{
    long lngOffset;
    char bytAttr;
    unsigned long lngUniqueID;
} pdbRecInfo;

/*** Cetus Record ***/
typedef struct
{
  unsigned char type;
  unsigned char readOnly;
  long          lat;
  long          lon;
  long          elevation;
  unsigned int  createdYear;
  unsigned char createdMonth;
  unsigned char createdDay;
  unsigned char createdHour;
  unsigned char createdMinute;
  unsigned char createdSecond;
  unsigned char sat;
  unsigned int  pdop;
  unsigned int  hdop;
  unsigned int  vdop;
  unsigned int  dgpsTime;
  unsigned long dgpsId;
  unsigned long averageTime;
  long          averageIte;
  unsigned int  dopMask;
  unsigned int  elevationMask;
  unsigned int  radius;
  unsigned long distance;
  unsigned int  visitedYear;
  unsigned char visitedMonth;
  unsigned char visitedDay;
  unsigned char visitedHour;
  unsigned char visitedMinute;
  unsigned char visitedSecond;
  unsigned char flagged;
  unsigned long icon;
  unsigned int  category;
  char          name[101];
  char          desc[101];
  char          data[101];
} GPSRecord;

/*** Options ***/
typedef union
{
  struct
  {
    unsigned char help:1;
    unsigned char verbose:1;
    unsigned char output:1;
    unsigned char unknown:1;

    unsigned char notitletab:1;
    unsigned char all:1;
    unsigned char natural:1;

    unsigned char dummy1:1;
    unsigned char dummy2;
  }Flg;
  unsigned int All;
} OptFlg;

/******************** <<PROTOTYPES>> *********************/
unsigned char GetUChar(FILE *fp);
int           GetInt(FILE *fp);
unsigned int  GetUInt(FILE *fp);
long          GetLong(FILE *fp);
unsigned long GetULong(FILE *fp);
void          GetString(FILE *fp, unsigned char *dst, int maxlen);
void          GetHeader(FILE *fp, pdbHeader* pHeader);
void          GetRecInfo(FILE *fp, pdbRecInfo *pRecInfo);

void          LoadPDBFile(FILE *fp);
void          TransRecord(FILE *fp_r, FILE *fp_w, OptFlg *options);
void          OutputT(FILE *out, char all, char natural);
void          Output(FILE *out,  GPSRecord GPSRec, char all, char natural);

int           search_options(char *string, OptFlg *options);
void          show_help(void);
FILE          *open_src(char *name);
FILE          *open_dst(char *name, char type);
void          execute(FILE *fp_src,FILE *fp_dst, OptFlg *options);


/******************** <<GLOBAL VAR>> *********************/
pdbHeader     Header;
pdbRecInfo    RecInfo[200];
unsigned long file_idx;
char          sep;
char          ext[5];

/********************** <<MAIN>> ************************/
int main(char argc, char *argv[])
{
  FILE *fp_src;
  FILE *fp_dst;
  char src_name[64];
  char dst_name[64];
  OptFlg *options;

  int i;
  int filename_idx;
  int error;

  options->All=0;
  filename_idx=0;
  error=0;
  sep='\t';
  ext[0]='.';
  ext[1]='x';
  ext[2]='l';
  ext[3]='s';
  ext[4]='\0';

  if (argc>NO_PAR)
  {
    // search for filenames and options parameters
    for(i=1;i<=argc;i++)
    {
      if(!search_options(argv[i], options))
       switch(filename_idx)
       {
	 case 0:
	  strcpy(src_name,argv[i]);
	  filename_idx++;
	  break;

	 case 1:
	  strcpy(dst_name,argv[i]);
	  filename_idx++;
	  break;

	 default:
	  filename_idx++;
	  break;
       }
    }
  }
  else
   // No parameters: show help set
   options->Flg.help=1;

  // unknow option: error
  if (options->Flg.unknown)
  {
    show_help();
    printf("\nERROR: unknown option\n");
    error=1;
  }

  if ( (options->Flg.help) && (!error) )
   // show only help
   show_help();
  else
  {
    // Open source file
    if ( (filename_idx>0) && (!error) )
    {
      fp_src = open_src(src_name);
      error=!fp_src;
    }

    // Open destination file
    if (!error)
    {
      if ((options->Flg.output) && (filename_idx>1) )
       fp_dst = open_dst(dst_name,DST_NAME);
      else
       fp_dst = open_dst(src_name,SRC_NAME);
      error=!fp_dst;
    }

    if (!error)
     execute(fp_src,fp_dst, options);

   }

  fclose(fp_src);
  fclose(fp_dst);

  return error;
}


/******************** <<FUNCTIONS>> **********************/
/*** Get Unsigned Char ***/
unsigned char GetUChar(FILE *fp)
{
  file_idx++;

  return fgetc(fp);
}


/*** Get Int ***/
int GetInt(FILE *fp)
{
  IntByte intTemp;

  intTemp.Byte[1]=fgetc(fp);
  intTemp.Byte[0]=fgetc(fp);

  file_idx+=2;

  return intTemp.All;
}

/*** Get Unsigned Int ***/
unsigned int GetUInt(FILE *fp)
{
  UIntByte uintTemp;

  uintTemp.Byte[1]=fgetc(fp);
  uintTemp.Byte[0]=fgetc(fp);

  file_idx+=2;

  return uintTemp.All;
}

/*** Get Long ***/
long GetLong(FILE *fp)
{
  LongByte longTemp;
  int i;

  for(i=0;i<4;i++)
   longTemp.Byte[3-i]=fgetc(fp);

  file_idx+=4;

  return longTemp.All;
}

/*** Get Unsigned Long ***/
unsigned long GetULong(FILE *fp)
{
  ULongByte ulongTemp;
  int i;

  for(i=0;i<4;i++)
   ulongTemp.Byte[3-i]=fgetc(fp);

  file_idx+=4;

  return ulongTemp.All;
}

/*** Get String ***/
void GetString(FILE *fp, unsigned char *dst, int maxlen)
{
  int i;
  char done;

  done=0;
  i=0;
  if (maxlen==0)
   while ((dst[i]=GetUChar(fp))!='\0')
    i++;
  else
   for(i=0;i<maxlen;i++)
   {
     if (!done)
     {
       dst[i]=GetUChar(fp);
       if (dst[i]=='\0')
	done=1;
     }
     else
      GetUChar(fp);
   }
}

/*** Get PDB Header ***/
void GetHeader(FILE *fp, pdbHeader* pHeader)
{
  GetString(fp, pHeader->strName, 32);
  pHeader->intAttr = GetInt(fp);
  pHeader->intVers = GetInt(fp);
  pHeader->lngCreate = GetLong(fp);
  pHeader->lngMod = GetLong(fp);
  pHeader->lngBackUp = GetLong(fp);
  pHeader->lngModNum = GetLong(fp);
  pHeader->lngAppInfoArea = GetLong(fp);
  pHeader->lngSortArea = GetLong(fp);
  GetString(fp, pHeader->strType , 4);
  pHeader->strType[4]='\0';
  GetString(fp, pHeader->strCreator , 4);
  pHeader->strCreator[4]='\0';
  pHeader->lngSeed = GetLong(fp);
  pHeader->lngNextRec = GetLong(fp);
  pHeader->intNumRecs = GetInt(fp);
}

/*** Get PDB Record Info ***/
void GetRecInfo(FILE *fp, pdbRecInfo *pRecInfo)
{
  ULongByte lngTemp;
  int i;

  pRecInfo->lngOffset = GetLong(fp);
  pRecInfo->bytAttr = (char)getc(fp);

  for(i=0;i<3;i++)
   lngTemp.Byte[i] = (char)getc(fp);
  lngTemp.Byte[3] = 0;

  pRecInfo->lngUniqueID = lngTemp.All;
}

/*** Get PDB File Info ***/
void LoadPDBFile(FILE *fp)
{
   int i;

   GetHeader(fp, &Header);
   for (i = 0; i < Header.intNumRecs; i++)
    GetRecInfo(fp, &RecInfo[i]);
}

/*** Get Cetus Record and translate it ***/
void TransRecord(FILE *fp_r,FILE *fp_w,OptFlg *options)
{
  GPSRecord GPSRec;

  GPSRec.type=GetUChar(fp_r);
  GPSRec.readOnly=GetUChar(fp_r);
  GPSRec.lat=GetLong(fp_r);
  GPSRec.lon=GetLong(fp_r);
  GPSRec.elevation=GetLong(fp_r);
  GPSRec.createdYear=GetUInt(fp_r);
  GPSRec.createdMonth=GetUChar(fp_r);
  GPSRec.createdDay=GetUChar(fp_r);
  GPSRec.createdHour=GetUChar(fp_r);
  GPSRec.createdMinute=GetUChar(fp_r);
  GPSRec.createdSecond=GetUChar(fp_r);
  GPSRec.sat=GetUChar(fp_r);
  GPSRec.pdop=GetUInt(fp_r);
  GPSRec.hdop=GetUInt(fp_r);
  GPSRec.vdop=GetUInt(fp_r);
  GPSRec.dgpsTime=GetUInt(fp_r);
  GPSRec.dgpsId=GetULong(fp_r);
  GPSRec.averageTime=GetULong(fp_r);
  GPSRec.averageIte=GetLong(fp_r);
  GPSRec.dopMask=GetUInt(fp_r);
  GPSRec.elevationMask=GetUInt(fp_r);
  GPSRec.radius=GetUInt(fp_r);
  GPSRec.distance=GetULong(fp_r);
  GPSRec.visitedYear=GetUInt(fp_r);
  GPSRec.visitedMonth=GetUChar(fp_r);
  GPSRec.visitedDay=GetUChar(fp_r);
  GPSRec.visitedHour=GetUChar(fp_r);
  GPSRec.visitedMinute=GetUChar(fp_r);
  GPSRec.visitedSecond=GetUChar(fp_r);
  GPSRec.flagged=GetUChar(fp_r);
  GPSRec.icon=GetULong(fp_r);
  GPSRec.category=GetUInt(fp_r);
  GetString(fp_r,GPSRec.name,0);
  GetString(fp_r,GPSRec.desc,0);
  GetString(fp_r,GPSRec.data,0);

  if (options->Flg.verbose)
   Output(stdout,GPSRec,options->Flg.all,options->Flg.natural);
  Output(fp_w,GPSRec,options->Flg.all,options->Flg.natural);
}

/*** Output Title ***/
void OutputT(FILE *out, char all, char natural)
{
  if (natural)
  {
    if (all)
    {
      fprintf(out,"type%c", sep);
      fprintf(out,"Ronly%c", sep);
    }
    fprintf(out,"lat%c", sep);
    fprintf(out,"lon%c", sep);
    fprintf(out,"elv%c", sep);
    fprintf(out,"cYr%c", sep);
    fprintf(out,"cMnt%c", sep);
    fprintf(out,"cDay%c", sep);
    fprintf(out,"cHr%c", sep);
    fprintf(out,"cMin%c", sep);
    fprintf(out,"cSec%c", sep);
    if (all)
    {
      fprintf(out,"sat%c", sep);
      fprintf(out,"pdop%c", sep);
      fprintf(out,"hdop%c", sep);
      fprintf(out,"vdop%c", sep);
      fprintf(out,"dgpsT%c", sep);
      fprintf(out,"dgpsId%c", sep);
    }
    fprintf(out,"aTime%c", sep);
    fprintf(out,"aIte%c", sep);
    if (all)
    {
      fprintf(out,"dopM%c", sep);
      fprintf(out,"elvM%c", sep);
      fprintf(out,"rad%c", sep);
      fprintf(out,"dis%c", sep);
      fprintf(out,"vYr%c", sep);
      fprintf(out,"vMnt%c", sep);
      fprintf(out,"vDay%c", sep);
      fprintf(out,"vHr%c", sep);
      fprintf(out,"vMin%c", sep);
      fprintf(out,"vSec%c", sep);
      fprintf(out,"flag%c", sep);
      fprintf(out,"icon%c", sep);
      fprintf(out,"cat%c", sep);
    }
    fprintf(out,"name%c", sep);
    fprintf(out,"desc%c", sep);
    if (all)
    {
      fprintf(out,"data");
    }
    fprintf(out,"\n");
  }
  else
  {
    if (all)
    {
      fprintf(out,"type%c", sep);
      fprintf(out,"Ronly%c", sep);
    }
    fprintf(out,"lat%c", sep);
    fprintf(out,"lon%c", sep);
    fprintf(out,"elv%c", sep);
    fprintf(out,"cDate%c", sep);
    fprintf(out,"cTime%c", sep);
    if (all)
    {
      fprintf(out,"sat%c", sep);
      fprintf(out,"pdop%c", sep);
      fprintf(out,"hdop%c", sep);
      fprintf(out,"vdop%c", sep);
      fprintf(out,"dgpsT%c", sep);
      fprintf(out,"dgpsId%c", sep);
    }
    fprintf(out,"aTime%c", sep);
    fprintf(out,"aIte%c", sep);
    if (all)
    {
      fprintf(out,"dopM%c", sep);
      fprintf(out,"elvM%c", sep);
      fprintf(out,"rad%c", sep);
      fprintf(out,"dis%c", sep);
      fprintf(out,"vDate%c", sep);
      fprintf(out,"vTime%c", sep);
      fprintf(out,"flag%c", sep);
      fprintf(out,"icon%c", sep);
      fprintf(out,"cat%c", sep);
    }
    fprintf(out,"name%c", sep);
    fprintf(out,"desc%c", sep);
    if (all)
    {
      fprintf(out,"data");
    }
    fprintf(out,"\n");

  }
}

/*** Output a Cetus Record ***/
void Output(FILE *out,  GPSRecord GPSRec, char all, char natural)
{
  double dbl_tmp;
  unsigned long lng_tmp;

  if (natural)
  {
    if (all)
    {
      fprintf(out,"%u", GPSRec.type);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.readOnly);
      fprintf(out,"%c",sep);
    }
    fprintf(out,"%ld",GPSRec.lat);
    fprintf(out,"%c",sep);
    fprintf(out,"%ld",GPSRec.lon);
    fprintf(out,"%c",sep);
    fprintf(out,"%ld",GPSRec.elevation);
    fprintf(out,"%c",sep);
    fprintf(out,"%u", GPSRec.createdYear);
    fprintf(out,"%c",sep);
    fprintf(out,"%u", GPSRec.createdMonth);
    fprintf(out,"%c",sep);
    fprintf(out,"%u", GPSRec.createdDay);
    fprintf(out,"%c",sep);
    fprintf(out,"%u", GPSRec.createdHour);
    fprintf(out,"%c",sep);
    fprintf(out,"%u", GPSRec.createdMinute);
    fprintf(out,"%c",sep);
    fprintf(out,"%u", GPSRec.createdSecond);
    fprintf(out,"%c",sep);
    if (all)
    {
      fprintf(out,"%u", GPSRec.sat);
      fprintf(out,"%c",sep);
      fprintf(out,"%d", GPSRec.pdop);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.hdop);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.vdop);
      fprintf(out,"%c",sep);
      fprintf(out,"%u",GPSRec.dgpsTime);
      fprintf(out,"%c",sep);
      fprintf(out,"%lu",GPSRec.dgpsId);
      fprintf(out,"%c",sep);
    }
    fprintf(out,"%lu",GPSRec.averageTime);
    fprintf(out,"%c",sep);
    fprintf(out,"%ld",GPSRec.averageIte);
    fprintf(out,"%c",sep);
    if (all)
    {
      fprintf(out,"%u", GPSRec.dopMask);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.elevationMask);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.radius);
      fprintf(out,"%c",sep);
      fprintf(out,"%lu",GPSRec.distance);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.visitedYear);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.visitedMonth);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.visitedDay);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.visitedHour);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.visitedMinute);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.visitedSecond);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.flagged);
      fprintf(out,"%c",sep);
      fprintf(out,"%lu",GPSRec.icon);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.category);
      fprintf(out,"%c",sep);
    }
    fprintf(out,"%s", GPSRec.name);
    fprintf(out,"%c",sep);
    fprintf(out,"%s", GPSRec.desc);
    fprintf(out,"%c",sep);
    if (all)
    {
      fprintf(out,"%s", GPSRec.data);
    }
    fprintf(out,"\n");
  }
  else
  {
    if (all)
    {
      switch(GPSRec.type)
      {
	case 0:
	 fprintf(out,"WptEdit");
	 break;
	case 1:
	 fprintf(out,"WptGPS2D");
	 break;
	case 2:
	 fprintf(out,"WptGPS3D");
	 break;
	case 3:
	 fprintf(out,"WptDGPS2D");
	 break;
	case 4:
	 fprintf(out,"WptDGPS3D");
	 break;
	case 5:
	 fprintf(out,"WptAverage");
	 break;
	case 6:
	 fprintf(out,"WptCache");
	 break;
	default:
	 break;
      }
      fprintf(out,"%c",sep);
      fprintf(out,"%s", GPSRec.readOnly?"YES":"NO");
      fprintf(out,"%c",sep);
    }
    dbl_tmp=((double)GPSRec.lat)/10000000;
    //fprintf(out,"%c%4.7f",(GPSRec.lat>0)?'N':'S',dbl_tmp);
    fprintf(out,"%4.7f",dbl_tmp);
    fprintf(out,"%c",sep);
    dbl_tmp=((double)GPSRec.lon)/10000000;
    //fprintf(out,"%c%4.7f",(GPSRec.lon>0)?'E':'W',dbl_tmp);
    fprintf(out,"%4.7f",dbl_tmp);
    fprintf(out,"%c",sep);
    dbl_tmp=((double)GPSRec.elevation)/100;
    fprintf(out,"%3.2f",dbl_tmp);
    fprintf(out,"%c",sep);
    //fprintf(out,"d %02u/%02u/%u ",GPSRec.createdDay,GPSRec.createdMonth,GPSRec.createdYear);
    fprintf(out,"%02u/%02u/%u",GPSRec.createdDay,GPSRec.createdMonth,GPSRec.createdYear);
    fprintf(out,"%c",sep);
    //fprintf(out,"h %02u:%02u.%02u",GPSRec.createdHour,GPSRec.createdMinute,GPSRec.createdSecond);
    fprintf(out,"%02u:%02u:%02u",GPSRec.createdHour,GPSRec.createdMinute,GPSRec.createdSecond);
    fprintf(out,"%c",sep);
    if (all)
    {
      fprintf(out,"%u", GPSRec.sat);
      fprintf(out,"%c",sep);
      fprintf(out,"%d", GPSRec.pdop);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.hdop);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.vdop);
      fprintf(out,"%c",sep);
      fprintf(out,"%u",GPSRec.dgpsTime);
      fprintf(out,"%c",sep);
      fprintf(out,"%lu",GPSRec.dgpsId);
      fprintf(out,"%c",sep);
    }
    lng_tmp=GPSRec.averageTime/3600;
    fprintf(out,"%02lu:",lng_tmp);
    lng_tmp=(GPSRec.averageTime%3600)/60;
    fprintf(out,"%02lu:",lng_tmp);
    lng_tmp=(GPSRec.averageTime%3600)%60;
    fprintf(out,"%02lu",lng_tmp);
    //fprintf(out,"%lu",GPSRec.averageTime);
    fprintf(out,"%c",sep);
    fprintf(out,"%ld",GPSRec.averageIte);
    fprintf(out,"%c",sep);
    if (all)
    {
      fprintf(out,"%u", GPSRec.dopMask);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.elevationMask);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.radius);
      fprintf(out,"%c",sep);
      fprintf(out,"%lu",GPSRec.distance);
      fprintf(out,"%c",sep);
      //fprintf(out,"d %02u/%02u/%u ",GPSRec.visitedDay,GPSRec.visitedMonth,GPSRec.visitedYear);
      fprintf(out,"%02u/%02u/%u",GPSRec.visitedDay,GPSRec.visitedMonth,GPSRec.visitedYear);
      fprintf(out,"%c",sep);
      //fprintf(out,"h %02u:%02u.%02u",GPSRec.visitedHour,GPSRec.visitedMinute,GPSRec.visitedSecond);
      fprintf(out,"%02u:%02u:%02u",GPSRec.visitedHour,GPSRec.visitedMinute,GPSRec.visitedSecond);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.flagged);
      fprintf(out,"%c",sep);
      fprintf(out,"%lu",GPSRec.icon);
      fprintf(out,"%c",sep);
      fprintf(out,"%u", GPSRec.category);
      fprintf(out,"%c",sep);
    }
    fprintf(out,"\"%s\"", GPSRec.name);
    fprintf(out,"%c",sep);
    fprintf(out,"\"%s\"", GPSRec.desc);
    fprintf(out,"%c",sep);
    if (all)
    {
      fprintf(out,"\"%s\"", GPSRec.data);
    }
    fprintf(out,"\n");
  }
}

/*** search options ***/
// search options, return 0 if not begins with '-'
int search_options(char *string, OptFlg *options)
{
  int i;

  i=0;
  if (string[0]=='-')
  {
    i=1;
    while(string[i]!='\0')
    {
      switch(string[i])
      {
	case 'h':
	case '?':
	 options->Flg.help=1;
	 break;

	case 'v':
	 options->Flg.verbose=1;
	 break;

	case 'o':
	 options->Flg.output=1;
	 break;

	case 'c':
	 sep=',';
	 ext[1]='t';
	 ext[2]='x';
	 ext[3]='t';
	 break;

	case 's':
	 sep=';';
	 ext[1]='c';
	 ext[2]='s';
	 ext[3]='v';
	 break;

	case 't':
	 sep='\t';
	 ext[1]='t';
	 ext[2]='x';
	 ext[3]='t';
	 break;

	case 'x':
	 sep='\t';
	 ext[1]='x';
	 ext[2]='l';
	 ext[3]='s';
	 break;

	case 'w':
	 options->Flg.notitletab=1;
	 break;

	case 'a':
	 options->Flg.all=1;
	 break;

	case 'n':
	 options->Flg.natural=1;
	 break;

	default:
	 options->Flg.unknown=1;
	 break;
      }
      i++;
    }
  }
  return i;
}

/*** show help ***/
void show_help(void)
{
  printf("\nCETPEXT v1.0 (March 19th 2003)\n");
  printf("\nCETus Position EXTractor: convert Cetus Position PDB file ");
  printf("(http://www.cetus.dk)\n");
  printf("into an easy readable data file\n");
  printf("\nAuthor:  Matteo Lenardon (e-mail: hangar18@virgilio.it)\n");
  printf("License: Freeware\n");
  printf("\nFormat: CETPEXT [-ovancstxwh] <sourcefile> [destinationfile]\n");
  printf("\nOptions:\n");
  printf("-o enable destination file name selection\n");
  printf("-v verbose\n");
  printf("-a show all data for records\n");
  printf("-n write file in the same format of Cetus pdb\n");
  printf("-c use a comma \",\" as separator (*.txt)\n");
  printf("-s use a semicolon \";\" as separator (*.csv)\n");
  printf("-t use a tab as separator (*.txt)\n");
  printf("-x use a tab as separator (*.xls) [default]\n");
  printf("-w write file without table title\n");
  printf("-h this help screen (disable other options)\n");
  printf("\n");
}

/*** open source file ***/
// open source file: if find '.' open "name." otherwise open "name.pdb"
FILE *open_src(char *name)
{
  FILE *fp;

  if(!strchr(name,'.'))
   fp=fopen(strcat(name,".pdb"),"rb");
  else
   fp=fopen(name,"rb");

  printf("\nCETPEXT\n");
  printf("src file: %s\n",name);

  if (!fp)
   printf("\nERROR: cannot open source file: %s\n",name);

  return fp;
}

/*** open destination file ***/
// open destination file:
//  if type == DST_NAME
//   open "name"
//  if type == SRC_NAME
//   open "name(without termination).xls"
FILE *open_dst(char *name, char type)
{
  FILE *fp;
  char *chp;

  int i;

  if (type==DST_NAME)
   fp=fopen(name,"w");
  else
  {
    if(!strchr(name,'.'))
     //fp=fopen(strcat(name,".xls"),"w");
     fp=fopen(strcat(name,ext),"w");
    else
    {
      chp=strchr(name,'.');
      /*
      chp[1]='x';
      chp[2]='l';
      chp[3]='s';
      chp[4]='\0';
      */
      for(i=0;i<5;i++)
       chp[i]=ext[i];

      fp=fopen(name,"w");
    }

  }

  printf("dst file: %s\n",name);

  if (!fp)
   printf("\nERROR: cannot open destination file: %s\n",name);

  return fp;
}

/*** main program ***/
void execute(FILE *fp_src,FILE *fp_dst, OptFlg *options)
{
  int i;

  LoadPDBFile(fp_src);
  if (!options->Flg.notitletab)
   OutputT(fp_dst,options->Flg.all,options->Flg.natural);
  fseek(fp_src,RecInfo[0].lngOffset,SEEK_SET);
  file_idx=RecInfo[0].lngOffset;
  for (i = 0; i < Header.intNumRecs; i++)
   TransRecord(fp_src,fp_dst,options);
}


